
/*
 * Copyright 2012 Rafaem M. Pestano.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.jsf.conventions.producer;

import com.jsf.conventions.qualifier.Property;
import java.io.IOException;
import java.io.Serializable;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.enterprise.context.ApplicationScoped;
import javax.enterprise.inject.Produces;
import javax.enterprise.inject.spi.InjectionPoint;
import javax.faces.context.FacesContext;
import javax.inject.Named;

/**
 *
 * @author Rafaem M. Pestano - Jul 7, 2012 7:03:43 PM
 * 
 */
@ApplicationScoped
@Named
public class PropertiesProvider implements Serializable {

    private Properties props = new Properties();

    public PropertiesProvider() {
        String propertiesPath = (String) FacesContext.getCurrentInstance().getExternalContext().getInitParameterMap().get("PROPERTIES_PATH");
        if (propertiesPath != null) {
            this.loadProperties(propertiesPath);
            Logger.getLogger(PropertiesProvider.class.getSimpleName()).log(Level.INFO, "Conventions properties provider loaded successfully.");
        }
    }

    public PropertiesProvider(String pathToFile) {
        loadProperties(pathToFile);
    }

    private void loadProperties(String pathToFile) {
        try {
            props.load(PropertiesProvider.class.getResourceAsStream(pathToFile));
        } catch (IOException e) {
            Logger.getLogger(PropertiesProvider.class.getSimpleName()).log(Level.WARNING, "Conventions could not load property file at " + pathToFile);
            e.printStackTrace();
        }
    }

    @Produces
    @Property
    public String produceConfigProperty(InjectionPoint ip) {
        
        if(ip.getAnnotated().isAnnotationPresent(Property.class)){
            Property property = ip.getAnnotated().getAnnotation(Property.class);
            if(!"".endsWith(property.file())){
                this.loadProperties(property.file());
            }
            String key = property.key();
            if (!"".endsWith(key)) {
                return props.getProperty(key);
            }
        }
        return "";
    }

    public Properties getProps() {
        return props;
    }

    public void setProps(Properties props) {
        this.props = props;
    }
    
}
